# -*- coding: utf-8 -*-
"""
Line String
"""

from typing import Any, ClassVar, List

from numpy import ndarray

from fudgeo.constant import DOUBLE, QUADRUPLE, TRIPLE
from fudgeo.geometry.base import AbstractGeometry
from fudgeo.geometry.point import Point, PointM, PointZ, PointZM
from fudgeo.geometry.util import Envelope


class BaseLineString(AbstractGeometry):
    """
    Base Line String
    """
    _class: ClassVar[Any]
    _dimension: ClassVar[int]
    _env_code: ClassVar[int]
    _has_m: ClassVar[bool]
    _has_z: ClassVar[bool]
    _wkb_prefix: ClassVar[bytes]
    _coordinates: ndarray

    def __init__(self, coordinates: List, srs_id: int) -> None: ...
    def __eq__(self, other: Any) -> bool: ...
    @property
    def coordinates(self) -> 'ndarray': ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def points(self) -> List: ...
    @property
    def envelope(self) -> Envelope: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> Any: ...
# End BaseLineString class


class LineString(BaseLineString):
    """
    Line String
    """
    def __init__(self, coordinates: List[DOUBLE], srs_id: int) -> None: ...
    def __eq__(self, other: 'LineString') -> bool: ...
    @property
    def coordinates(self) -> 'ndarray': ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def points(self) -> List[Point]: ...
    @property
    def envelope(self) -> Envelope: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> 'LineString': ...
# End LineString class


class LineStringZ(BaseLineString):
    """
    Line String Z
    """
    def __init__(self, coordinates: List[TRIPLE], srs_id: int) -> None: ...
    def __eq__(self, other: 'LineStringZ') -> bool: ...
    @property
    def coordinates(self) -> 'ndarray': ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def points(self) -> List[PointZ]: ...
    @property
    def envelope(self) -> Envelope: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> 'LineStringZ': ...
# End LineStringZ class


class LineStringM(BaseLineString):
    """
    Line String M
    """
    def __init__(self, coordinates: List[TRIPLE], srs_id: int) -> None: ...
    def __eq__(self, other: 'LineStringM') -> bool: ...
    @property
    def coordinates(self) -> 'ndarray': ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def points(self) -> List[PointM]: ...
    @property
    def envelope(self) -> Envelope: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> 'LineStringM': ...
# End LineStringM class


class LineStringZM(BaseLineString):
    """
    Line String ZM
    """
    def __init__(self, coordinates: List[QUADRUPLE], srs_id: int) -> None: ...
    def __eq__(self, other: 'LineStringZM') -> bool: ...
    @property
    def coordinates(self) -> 'ndarray': ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def points(self) -> List[PointZM]: ...
    @property
    def envelope(self) -> Envelope: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> 'LineStringZM': ...
# End LineStringZM class


class BaseMultiLineString(AbstractGeometry):
    """
    Base Multi Line String
    """
    _class: ClassVar[Any]
    _dimension: ClassVar[int]
    _env_code: ClassVar[int]
    _wkb_prefix: ClassVar[bytes]
    _lines: List

    def __init__(self, coordinates: List[List], srs_id: int) -> None: ...
    def __eq__(self, other: Any) -> bool: ...
    def _make_lines(self, coordinates: List[List]) -> List: ...
    @property
    def lines(self) -> List: ...
    @property
    def is_empty(self) -> bool: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @property
    def envelope(self) -> Envelope: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> Any: ...
# End BaseMultiLineString class


class MultiLineString(BaseMultiLineString):
    """
    Multi Line String
    """
    def __init__(self, coordinates: List[List[DOUBLE]], srs_id: int) -> None: ...
    def __eq__(self, other: 'MultiLineString') -> bool: ...
    def _make_lines(self, coordinates: List[List[DOUBLE]]) -> List[LineString]: ...
    @property
    def lines(self) -> List[LineString]: ...
    @property
    def is_empty(self) -> bool: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @property
    def envelope(self) -> Envelope: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> 'MultiLineString': ...
# End MultiLineString class


class MultiLineStringZ(BaseMultiLineString):
    """
    Multi Line String Z
    """
    def __init__(self, coordinates: List[List[TRIPLE]], srs_id: int) -> None: ...
    def __eq__(self, other: 'MultiLineStringZ') -> bool: ...
    def _make_lines(self, coordinates: List[List[TRIPLE]]) -> List[LineStringZ]: ...
    @property
    def lines(self) -> List[LineStringZ]: ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def envelope(self) -> Envelope: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> 'MultiLineStringZ': ...
# End MultiLineStringZ class


class MultiLineStringM(BaseMultiLineString):
    """
    Multi Line String M
    """
    def __init__(self, coordinates: List[List[TRIPLE]], srs_id: int) -> None: ...
    def __eq__(self, other: 'MultiLineStringM') -> bool: ...
    def _make_lines(self, coordinates: List[List[TRIPLE]]) -> List[LineStringM]: ...
    @property
    def lines(self) -> List[LineStringM]: ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def envelope(self) -> Envelope: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> 'MultiLineStringM': ...
# End MultiLineStringM class


class MultiLineStringZM(BaseMultiLineString):
    """
    Multi Line String ZM
    """
    def __init__(self, coordinates: List[List[QUADRUPLE]], srs_id: int) -> None: ...
    def __eq__(self, other: 'MultiLineStringZM') -> bool: ...
    def _make_lines(self, coordinates: List[List[QUADRUPLE]]) -> List[LineStringZM]: ...
    @property
    def lines(self) -> List[LineStringZM]: ...
    @property
    def is_empty(self) -> bool: ...
    @property
    def envelope(self) -> Envelope: ...
    def _to_wkb(self, ary: bytearray) -> bytearray: ...
    @classmethod
    def from_gpkg(cls, value: bytes) -> 'MultiLineStringZM': ...
# End MultiLineStringZM class


if __name__ == '__main__':  # pragma: no cover
    pass
